/*******************************************************************************
 * Copyright (C) 2017 Philipp Spohr
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 ******************************************************************************/
package de.hhu.ba.yoshikoWrapper.core;

import java.io.File;


import org.slf4j.Logger;

import de.hhu.ba.yoshikoWrapper.logging.YoshikoLogger;
import de.hhu.ba.yoshikoWrapper.swig.LibraryInterface;

public class YoshikoLoader {

	public static final String REQUIRED_VERSION = "2.1.1";

	private static boolean isLoaded;

	//Symbolic Links
	private static Logger logger = YoshikoLogger.getInstance().getLogger();

	public static void loadLibrary(String libPath) {
		//Attempt to load from a previously stored path
		File f = new File (libPath);
		if (!f.exists()) {
			logger.info("Could not load the Yoshiko-library from a previously saved path! You might need to set the path manually.");
			return;
		}

		try {
			logger.info("Attempting to load library @: "+libPath);
			//Verify version ending
			if (!libPath.substring(0,libPath.lastIndexOf(".")).endsWith(REQUIRED_VERSION)) {
				logger.info(LocalizationManager.get("libFail")+" "+REQUIRED_VERSION);
				return;
			}
			//Perform actual loading
			System.load(libPath);

		}
		catch(UnsatisfiedLinkError e) {
			//This might happen as Java doesn't unload native libs
			//One such case might be reloading the Cytoscape app on-the-fly
			logger.error(e.getMessage());
		}
		LibraryInterface.setVerbosity(3);
		//update cfg
		CyCore.cm.getProperties().setProperty("pathToYoshiko", libPath);
		isLoaded = true;
	}

	//SETTER / GETTER

	public static boolean isLibraryLoaded() {
		return isLoaded;
	}

}
